package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/constants"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/format"
	"fusenapi/utils/s3url_to_s3id"
	"gorm.io/gorm"
	"strings"

	"context"

	"fusenapi/server/product/internal/svc"
	"fusenapi/server/product/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetRecommendProductListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetRecommendProductListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetRecommendProductListLogic {
	return &GetRecommendProductListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetRecommendProductListLogic) GetRecommendProductList(req *types.GetRecommendProductListReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.Num > 100 || req.Num < 0 {
		req.Num = 4
	}
	productInfo, err := l.svcCtx.AllModels.FsProduct.FindOne(l.ctx, req.ProductId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "detail`s product is not found")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get detail product info")
	}
	var (
		recommendProductList []gmodel.FsProduct
	)
	if productInfo.RecommendProduct != nil && *productInfo.RecommendProduct != "" {
		recommendProductIds, err := format.StrSlicToInt64Slice(strings.Split(*productInfo.RecommendProduct, ","))
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to split recommend product ids")
		}
		recommendProductList, err = l.svcCtx.AllModels.FsProduct.GetProductListByIds(l.ctx, recommendProductIds, "")
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get recommend product list")
		}
		if len(recommendProductList) > int(req.Num) {
			recommendProductList = recommendProductList[:req.Num]
		}
	}
	//资源id集合
	resourceIds := make([]string, 0, 50)
	//需要填充时需要忽略的id
	ignoreProductIds := make([]int64, 0, len(recommendProductList))
	productIds := make([]int64, 0, len(recommendProductList))
	//在合并之前记住推荐的产品
	mapRecommend := make(map[int64]struct{})
	for _, v := range recommendProductList {
		ignoreProductIds = append(ignoreProductIds, v.Id)
		productIds = append(productIds, v.Id)
		mapRecommend[v.Id] = struct{}{}
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.CoverImg))
	}
	//小于请求的数量则需要从产品表中随机填补上(不包含上面的产品)
	lenRecommendProduct := len(recommendProductList)
	if lenRecommendProduct < int(req.Num) {
		appendNum := int(req.Num) - lenRecommendProduct
		productList, err := l.svcCtx.AllModels.FsProduct.GetIgnoreRandomProductList(l.ctx, appendNum, ignoreProductIds)
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get product list")
		}
		//合并列表
		for _, v := range productList {
			productIds = append(productIds, v.Id)
			recommendProductList = append(recommendProductList, v)
		}
	}
	//获取商品可选配件
	productOptionalPartList, err := l.svcCtx.AllModels.FsProductModel3d.GetGroupPartListByProductIds(l.ctx, productIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get product part list")
	}
	//存储有配件的map
	mapProductHaveOptionFitting := make(map[int64]struct{})
	for _, partList := range productOptionalPartList {
		partList.PartList = strings.Trim(partList.PartList, " ")
		partList.PartList = strings.Trim(partList.PartList, ",")
		if partList.PartList == "" {
			continue
		}
		mapProductHaveOptionFitting[partList.ProductId] = struct{}{}
	}
	//获取产品尺寸数量
	productSizeCountList, err := l.svcCtx.AllModels.FsProductSize.GetGroupProductSizeByStatus(l.ctx, productIds, 1)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get ")
	}
	mapProductSizeCount := make(map[int64]int64)
	for _, v := range productSizeCountList {
		mapProductSizeCount[v.ProductId] = v.Num
	}
	//获取产品最低价
	mapProductMinPrice := make(map[int64]int64)
	modelList, err := l.svcCtx.AllModels.FsProductModel3d.GetAllByProductIdsTags(l.ctx, []int64{productInfo.Id}, []int{constants.TAG_MODEL, constants.TAG_PARTS}, "id,size_id,product_id,price,tag,part_id,step_price")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get model list")
	}
	if err = l.svcCtx.AllModels.FsProductModel3d.GetProductMinPrice(modelList, mapProductMinPrice); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get product min price")
	}
	//获取产品标签相关属性
	productTagPropList, err := l.svcCtx.AllModels.FsProductTagProp.GetTagPropByProductIdsWithProductTag(l.ctx, productIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get product tag property")
	}
	for _, v := range productTagPropList {
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
	}
	//根据resourceUrls找到对应的元数据
	resourceMetadataList, err := l.svcCtx.AllModels.FsResource.FindAllByResourceIds(l.ctx, resourceIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get resource list")
	}
	mapResourceMetadata := make(map[string]map[string]interface{})
	for _, v := range resourceMetadataList {
		var metadata map[string]interface{}
		if v.Metadata != nil {
			_ = json.Unmarshal(*v.Metadata, &metadata)
		}
		mapResourceMetadata[*v.ResourceUrl] = metadata
	}
	mapTagProp := make(map[int64][]types.CoverDefaultItem)
	for _, v := range productTagPropList {
		mapTagProp[*v.ProductId] = append(mapTagProp[*v.ProductId], types.CoverDefaultItem{
			TemplateTag:   v.TemplateTag,
			Cover:         *v.Cover,
			CoverMetadata: mapResourceMetadata[*v.Cover],
		})
	}
	list := make([]types.GetRecommandProductListRsp, 0, len(recommendProductList))
	for _, v := range recommendProductList {
		recommend := false
		if _, ok := mapRecommend[v.Id]; ok {
			recommend = true
		}
		minPrice := int64(0)
		if minVal, ok := mapProductMinPrice[v.Id]; ok {
			minPrice = minVal
		}
		sizeCount := int64(0)
		if sc, ok := mapProductSizeCount[v.Id]; ok {
			sizeCount = sc
		}
		haveOptionalFitting := false
		if _, ok := mapProductHaveOptionFitting[v.Id]; ok {
			haveOptionalFitting = true
		}
		item := types.GetRecommandProductListRsp{
			Id:                  v.Id,
			Sn:                  *v.Sn,
			Title:               *v.Title,
			TitleCn:             *v.TitleCn,
			Cover:               *v.Cover,
			CoverMetadata:       mapResourceMetadata[*v.Cover],
			CoverImg:            *v.CoverImg,
			CoverImgMetadata:    mapResourceMetadata[*v.CoverImg],
			CoverDefault:        []types.CoverDefaultItem{},
			Intro:               *v.Intro,
			Recommend:           recommend,
			MinPrice:            minPrice,
			IsCustomization:     *v.IsCustomization,
			SizeCount:           sizeCount,
			HaveOptionalFitting: haveOptionalFitting,
		}
		if _, ok := mapTagProp[productInfo.Id]; ok {
			item.CoverDefault = mapTagProp[productInfo.Id]
		}
		list = append(list, item)
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", list)
}
